@@ -10,6 +10,7 @@ import glob
 import logging
 import io
 import urllib.parse
+import traceback
 
 import dqueue.core 
 import dqueue.app
@@ -62,11 +63,14 @@ logger=logging.getLogger(__name__)
 
 ## === schemas
 
+class TaskData(Schema):
+    pass
+
 class Task(Schema):
     state = fields.Str()
     queue = fields.Str()
-    task_id = fields.Str()
-    task_data = fields.Dict()
+    task_key = fields.Str()
+    task_data = TaskData
 
 class TaskList(Schema):
     tasks = fields.Nested(Task, many=True)
@@ -171,6 +175,68 @@ app.add_url_rule(
           methods=['GET']
 )
 
+class WorkerAnswer(SwaggerView):
+    operationId = "answer"
+
+    parameters = [
+                {
+                    'name': 'worker_id',
+                    'in': 'query',
+                    'required': True,
+                    'type': 'string',
+                },
+                {
+                    'name': 'queue',
+                    'in': 'query',
+                    'required': False,
+                    'type': 'string',
+                },
+                {
+                    'name': 'token',
+                    'in': 'query',
+                    'required': True,
+                    'type': 'string',
+                },
+                {
+                    'name': 'task_dict',
+                    'in': 'body',
+                    'required': True,
+                    'schema': Task, 
+                },
+            ]
+
+    responses = {
+            200: {
+                    'description': 'its ok',
+                    'schema': Task,
+                 }
+            }
+
+    def post(self):
+        queue = dqueue.core.Queue(request.args.get('queue', 'default'))
+
+        task_dict = request.json
+
+        logger.debug("setting current task in %s to %s", queue, task_dict)
+
+        queue.state = "done"
+        queue.current_task = dqueue.core.Task.from_entry(task_dict)
+        queue.current_task_stored_key = queue.current_task.key
+        task = queue.current_task
+
+        queue.task_done()
+
+        return jsonify(
+                    { 'task_key': task.key, **task.as_dict}
+               )
+
+
+app.add_url_rule(
+         '/worker/answer',
+          view_func=WorkerAnswer.as_view('worker_answer_task'),
+          methods=['POST']
+)
+
 class WorkerQuestion(SwaggerView):
     operationId = "questionTask"
 
@@ -185,7 +251,7 @@ class WorkerQuestion(SwaggerView):
                     'name': 'task_data',
                     'in': 'body',
                     'required': True,
-                    'schema': Task, 
+                    'schema': TaskData,
                 },
                 {
                     'name': 'token',
@@ -203,7 +269,8 @@ class WorkerQuestion(SwaggerView):
 
     responses = {
             200: {
-                    'description': 'task data',
+                    'description': 'task dict',
+                    'schema': Task
                 }
         }
 
@@ -216,12 +283,12 @@ class WorkerQuestion(SwaggerView):
 
         print("got:", worker_id, task_data)
 
-        task = queue.put(task_data)
+        task_entry = queue.put(task_data)
 
-        logger.warning("questioned task: %s", task)
+        logger.warning("questioned task: %s", task_entry)
         return jsonify(
-                {}
-            )
+                    task_entry
+                )
 
 app.add_url_rule(
      '/worker/question',
@@ -232,7 +299,7 @@ app.add_url_rule(
 class TaskView(SwaggerView):
     parameters = [
                 {
-                    'name': 'task_id',
+                    'name': 'task_key',
                     'in': 'path',
                     'required': True,
                     'type': 'string',
@@ -246,11 +313,11 @@ class TaskView(SwaggerView):
                 }
         }
 
-    def get(self, task_id):
-        info = tools.task_info(task_id)
-        logger.warning("requested task_id %s %s", task_id, info)
+    def get(self, task_key):
+        info = tools.task_info(task_key)
+        logger.warning("requested task_key %s %s", task_key, info)
         return jsonify(
-                task_id=task_id,
+                task_key=task_key,
                 task_info=info,
             )
 
@@ -341,13 +408,19 @@ def tasks_purge():
 
     queue = dqueue.core.Queue()
     n = queue.purge()
+
     return jsonify(
             nentries=n
         )
 
+@app.errorhandler(Exception)
+def handle(error):
+    logger.error("error: %s", repr(error))
+
+    traceback.print_exc()
 
 app.add_url_rule(
-         '/task/view/<task_id>',
+         '/task/view/<task_key>',
           view_func=TaskView.as_view('api_task'),
           methods=['GET']
 )
