@@ -25,6 +25,11 @@ try:
 except ImportError:
     import urllib.parse as urlparse# type: ignore
 
+from typing import NewType, Dict
+
+TaskDict = NewType('TaskDict', Dict)
+TaskData = NewType('TaskData', Dict)
+
 import pymysql
 import peewee # type: ignore
 from playhouse.db_url import connect # type: ignore
@@ -101,7 +106,7 @@ except peewee.OperationalError:
 except Exception:
     has_mysql = False
 
-class Task(object):
+class Task:
     def __init__(self,task_data,execution_info=None, submission_data=None, depends_on=None):
         self.task_data=task_data
         self.submission_info=self.construct_submission_info()
@@ -159,9 +164,12 @@ class Task(object):
         return self.get_key(True)
 
     def get_key(self,key=True):
-        components=[]
+        components = []
 
-        task_data_string=yaml.dump(self.task_data,encoding='utf-8')
+        task_data_string = yaml.dump(order_task_data(self.task_data), encoding='utf-8')
+
+        logger.debug("task data: %s", self.task_data)
+        logger.debug("task data string: %s", task_data_string)
 
         components.append(sha224(task_data_string).hexdigest()[:8])
         #log("encoding: "+repr(components))
@@ -175,11 +183,18 @@ class Task(object):
 
             components.append(sha224(str(OrderedDict(sorted(self.submission_info.items()))).encode('utf-8')).hexdigest()[:8])
 
-        return "_".join(components)
+        key = "_".join(components)
+
+        logger.warning("generating key %s from %s", key, task_data_string)
+
+        return key
 
     def __repr__(self):
         return "[{}: {}]".format(self.__class__.__name__,self.task_data)
 
+    def filename_instance(self):
+        return "unset"
+
 def makedir_if_neccessary(directory):
     try:
         os.makedirs(directory)
@@ -194,6 +209,16 @@ def list_queues(pattern=None):
         return [ Queue(task_entry.queue) for task_entry in 
                 TaskEntry.select(TaskEntry.queue).where(TaskEntry.queue % pattern).distinct(TaskEntry.queue) ]
 
+
+def order_task_data(d):
+    if isinstance(d, dict) or isinstance(d, OrderedDict) or isinstance(d, defaultdict):
+        return OrderedDict({
+                k:order_task_data(v) for k, v in sorted(d.items())
+            })
+
+    return d
+
+
 class Queue:
     def list_queues(self, pattern=None):
         return list_queues(pattern)
@@ -271,10 +296,10 @@ class Queue:
         assert len(r)==1
         return r[0]
     
-    def put(self,task_data,submission_data=None, depends_on=None):
-        ""
+    def put(self, task_data: TaskData, submission_data=None, depends_on=None) -> TaskDict:
+        logger.info("putting in queue task_data %s", task_data)
 
-        assert depends_on is None or type(depends_on) in [list,tuple]
+        assert depends_on is None or type(depends_on) in [list, tuple]
 
         task=Task(task_data,submission_data=submission_data,depends_on=depends_on)
 
@@ -302,7 +327,9 @@ class Queue:
         if instance_for_key is not None:
             log("found existing instance(s) for this key, no need to put:",instances_for_key)
             self.log_task("task already found",task,instance_for_key['state'])
-            return instance_for_key
+            d = model_to_dict(instance_for_key['task_entry'])
+            log("task entry:", d)
+            return d
 
         if depends_on is None:
             self.insert_task_entry(task,"waiting")
@@ -319,19 +346,23 @@ class Queue:
             log("stored:",task.filename_instance)
             log("recovered:", recovered_task.filename_instance)
     
-            nfn=self.queue_dir("conflict") + "/put_original_" + task.filename_instance
-            open(nfn, "w").write(task.serialize())
+            #nfn=self.queue_dir("conflict") + "/put_original_" + task.filename_instance
+            #open(nfn, "w").write(task.serialize())
         
-            nfn=self.queue_dir("conflict") + "/put_recovered_" + recovered_task.filename_instance
-            open(nfn, "w").write(recovered_task.serialize())
+            #nfn=self.queue_dir("conflict") + "/put_recovered_" + recovered_task.filename_instance
+            #open(nfn, "w").write(recovered_task.serialize())
             
-            nfn=self.queue_dir("conflict") + "/put_stored_" + os.path.basename(fn)
-            open(nfn, "w").write(open(fn).read())
+            #nfn=self.queue_dir("conflict") + "/put_stored_" + os.path.basename(fn)
+            #open(nfn, "w").write(open(fn).read())
 
             raise Exception("Inconsistent storage")
 
         log("successfully put in queue:",instance_for_key['task_entry'].entry)
-        return dict(state="submitted",task_entry=instance_for_key['task_entry'].entry)
+
+        #return dict(state="submitted", task_entry=instance_for_key['task_entry'].entry)
+        d = model_to_dict(instance_for_key['task_entry'])
+        d['state'] = 'submitted'
+        return d
 
     def get(self):
         ""
@@ -370,14 +401,14 @@ class Queue:
 
         if self.current_task.key != entry.key:
             log("inconsitent storage:")
-            log(">>>> stored:", task_name)
-            log(">>>> recovered:", self.current_task.filename_instance)
+            log(">>>> stored:", entry)
+            log(">>>> recovered:", self.current_task)
 
-            fn=self.queue_dir("conflict") + "/get_stored_" + self.current_task.filename_instance
-            open(fn, "w").write(self.current_task.serialize())
+            #fn=self.queue_dir("conflict") + "/get_stored_" + self.current_task.filename_instance
+            #open(fn, "w").write(self.current_task.serialize())
         
-            fn=self.queue_dir("conflict") + "/get_recovered_" + task_name
-            open(fn, "w").write(open(self.queue_dir("waiting")+"/"+task_name).read())
+            #fn=self.queue_dir("conflict") + "/get_recovered_" + task_name
+            #open(fn, "w").write(open(self.queue_dir("waiting")+"/"+task_name).read())
 
             raise Exception("Inconsistent storage")
 
@@ -410,6 +441,8 @@ class Queue:
         nentries=TaskEntry.delete().execute()
         log("deleted %i"%nentries)
 
+        return nentries
+
     
     def try_all_locked(self):
         ""
@@ -424,8 +457,8 @@ class Queue:
     def remember(self,task_data,submission_data=None):
         ""
         task=Task(task_data,submission_data=submission_data)
-        nfn=self.queue_dir("problem") + "/"+task.filename_instance
-        open(nfn, "w").write(task.serialize())
+        #nfn=self.queue_dir("problem") + "/"+task.filename_instance
+        #open(nfn, "w").write(task.serialize())
             
 
     def insert_task_entry(self,task,state):
@@ -632,7 +665,7 @@ class Queue:
     def watch(self,delay=1):
         """"""
         while True:
-            log(self.info())
+            log(self.info)
             time.sleep(delay)
 
     def get_worker_id(self):
@@ -647,7 +680,7 @@ class Queue:
         return "{fqdn}.{pid}".format(**d)
 
 
-    def log_task(self,message,task=None,state=None):
+    def log_task(self, message, task=None, state='undefined'):
         ""
         if task is None:
             task=self.current_task
